#include "orbitersdk.h"

// Calculate lift coefficient [Cl] as a function of aoa (angle of attack) over -Pi ... Pi
// Implemented here as a piecewise linear function
double LiftCoeff(double aoa)
{
	int i;
	const int nlift = 9;
	static const double AOA[nlift] = { -180 * RAD,-60 * RAD,-30 * RAD,-1 * RAD,15 * RAD,20 * RAD,25 * RAD,60 * RAD,180 * RAD };
	static const double CL[nlift] = { 0,      0,   -0.1,     0,   0.2,  0.25,   0.2,     0,      0 };
	static const double SCL[nlift] = { (CL[1] - CL[0]) / (AOA[1] - AOA[0]), (CL[2] - CL[1]) / (AOA[2] - AOA[1]),
									  (CL[3] - CL[2]) / (AOA[3] - AOA[2]), (CL[4] - CL[3]) / (AOA[4] - AOA[3]),
									  (CL[5] - CL[4]) / (AOA[5] - AOA[4]), (CL[6] - CL[5]) / (AOA[6] - AOA[5]),
									  (CL[7] - CL[6]) / (AOA[7] - AOA[6]), (CL[8] - CL[7]) / (AOA[8] - AOA[7]) };
	for (i = 0; i < nlift - 1 && AOA[i + 1] < aoa; i++);
	return CL[i] + (aoa - AOA[i]) * SCL[i];
}

typedef struct {
	VECTOR3 Location;
	double span;
	double S;
	double AR;
} AIRFOIL;

class Velcro : public VESSEL3 {
public:
	Velcro(OBJHANDLE hVessel, int flightmodel);
	~Velcro();
	void clbkSetClassCaps(FILEHANDLE cfg);
	void ScanLines(FILEHANDLE cfg, void* vs);
	void SetVelcroStage();
	void MakeExhausts();
	void SetAttControlsVELCRO();
	void adjust_td();
	void clbkLoadStateEx(FILEHANDLE scn, void* vs);
	void clbkSaveState(FILEHANDLE scn);
	int  clbkConsumeBufferedKey(DWORD key, bool down, char* kstate);
	void clbkPreStep(double simt, double simdt, double mjd);
	void clbkPostStep(double simt, double simdt, double mjd);

	double padbias;
	double display_padbias_time;
	VECTOR3 p1, p2, p3;

	bool BecomeSomethingElse, init = false;
	bool is_multistage;
	char meshname[256], becomename[256];
	double EngineThrust, IspSL, IspVac, RCSThrust, Mass, MaxFuelMass, mayfly;
	VECTOR3 MEngineRef[64];
	VECTOR3 MEngineVec;
	VECTOR3 CAMERA_OFS;
	int numexhausts, numengines;
	AIRFOIL Wing, HStab, VStab;

	PROPELLANT_HANDLE ph_first;

	THRUSTER_HANDLE   th_first[64], th_dummy[1];
	THGROUP_HANDLE    thg_first, thg_dummy;
	SURFHANDLE		  tex, texstor, texfluor, texkero;

	enum ExhaustType { SRM, N2O4, LF2, LH2, KERO, HYBRID } ExhaustType;
};
